/*
 *  Interface with Fossil driver by MaxWolf (2:5020/299)
 *  Based on the Arjen G. Lentz (2:283/512) HYDRACOM source code
 */

#define _COM_
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <dos.h>
#include "com.h"

/* implemented:
void dtr_out (byte flag)
void com_flow (byte flags)
void com_setspeed (word speed)
static word fos_getstatus (void)
static int get_fos_info (void)
static int fos_fillinbuf (void)
void com_putblock (byte *s, word len)
static int fos_init (void)
void sys_init(void)         /* all system dependent init should be done here */
void sys_reset(void)        /* same as above for de-init */
int com_outfull (void)
int com_infull (void)
int carrier(void)
void com_flush(void)        /* wait till all characters have been sent */
void com_putbyte (byte c)
void com_purge (void)
void com_dump (void)
int com_getbyte(void)
*/


struct _fos_info {
       word      strsize;       /* size of the structure in bytes     */
       byte      majver;        /* FOSSIL spec driver conforms to     */
       byte      minver;        /* rev level of this specific driver  */
       char far *ident;         /* FAR pointer to ASCII ID string     */
       word      ibufr;         /* size of the input buffer (bytes)   */
       word      ifree;         /* number of bytes left in buffer     */
       word      obufr;         /* size of the output buffer (bytes)  */
       word      ofree;         /* number of bytes left in the buffer */
       byte      swidth;        /* width of screen on this adapter    */
       byte      sheight;       /* height of screen    "      "       */
       byte      speed;         /* ACTUAL speed, computer to modem    */
};

struct _fos_speedtable {
        word speed;
        byte bits;
};

#define fossil_int(func)   { _DX = port; _AH = func; geninterrupt(0x14); }
#define dosext_int(func)   { _AH = func;             geninterrupt(0x15); }
#define dos_int(func)      { _AH = func;             geninterrupt(0x21); }
#define dossched_int()     {                         geninterrupt(0x28); }
#define FOS_SIGNATURE   0x1954
#define FOS_SETSPEED    0x00
#define FOS_PUTC        0x01
#define FOS_GETSTATUS   0x03
#define FOS_INIT        0x04
#define FOS_DEINIT      0x05
#define FOS_SETDTR      0x06
#define FOS_DUMP        0x09
#define FOS_PURGE       0x0a
#define FOS_SETFLOW     0x0f
#define FOS_MAXBUFLEN   1024

static struct _fos_info fos_info;
static byte    fos_inbuf[FOS_MAXBUFLEN];
static word    fos_inwrite,
			   fos_inread;

static struct _fos_speedtable fos_speedtable [] = {
        {   300U, 0x40 },
        {  1200U, 0x80 },
        {  2400U, 0xA0 },
        {  4800U, 0xC0 },
        {  9600U, 0xE0 },
        { 19200U, 0x00 },
        { 38400U, 0x20 },
        {     0U,    0 }
};


void dtr_out (byte flag)
{
           _AL = flag;
           fossil_int(FOS_SETDTR);
}


void com_flow (byte flags)
{
           _AL = flags;
           fossil_int(FOS_SETFLOW);
}


void com_setspeed (word speed)  /* set speed of comport */
{
        register byte i;

        if (!speed) return;
           for (i = 0; fos_speedtable[i].speed; i++) {
               if (speed == fos_speedtable[i].speed) {
                  _AL = (fos_speedtable[i].bits | (parity ? 0x1a : 0x03));
                  fossil_int(FOS_SETSPEED);
                  break;
               }
           }
}


static word fos_getstatus (void)
{
        fossil_int(FOS_GETSTATUS);
        return (_AX);
}


static int get_fos_info (void)
{
        struct SREGS sregs;
        union  REGS  regs;

        regs.x.ax = 0x1b00;
        regs.x.cx = sizeof (struct _fos_info);
        regs.x.dx = port;
        segread(&sregs);
        sregs.es  = FP_SEG(&fos_info);
        regs.x.di = FP_OFF(&fos_info);
        int86x(0x14,&regs,&regs,&sregs);

        return (regs.x.ax == sizeof (struct _fos_info));
}


static int fos_fillinbuf (void)
{
		struct SREGS sregs;
		union  REGS  regs;

        if (!(fos_getstatus() & 0x0100))
           return (0);

		regs.x.ax = 0x1800;
        regs.x.dx = port;
        regs.x.cx = FOS_MAXBUFLEN;
        segread(&sregs);
        sregs.es  = FP_SEG(fos_inbuf);
        regs.x.di = FP_OFF(fos_inbuf);
        int86x(0x14,&regs,&regs,&sregs);
        fos_inwrite = regs.x.ax;
        fos_inread = 0;

        return (fos_inwrite);
}

int carrier(void)  /* Return carrier status */
{
        int i;
           fossil_int(FOS_GETSTATUS);
           i = _AX;
           return(i & dcdmask);
}


void com_putblock (byte *s, word len)
{
           struct SREGS sregs;
           union  REGS  regs;

		   while (len) {
                 regs.x.ax = 0x1900;
                 regs.x.dx = port;
                 regs.x.cx = len;
                 segread(&sregs);
                 sregs.es  = FP_SEG(s);
                 regs.x.di = FP_OFF(s);
                 int86x(0x14,&regs,&regs,&sregs);
                 len -= regs.x.ax;
                 s += regs.x.ax;
           }
}


static int fos_init (void)
{
        fossil_int(FOS_INIT);
        return ((_AX != FOS_SIGNATURE || _BH < 5 || _BL < 0x1b) ? 0 : 1);
}


void sys_init(void) /* all system dependent init should be done here */
{
		if (!fos_init()) {
			fprintf(stderr, "Fossil not loaded !\n");
			exit(1);
		}
		if (!get_fos_info() || fos_info.majver < 5) {
			fossil_int(FOS_DEINIT);
			fprintf(stderr,"It requires a revision 5 FOSSIL driver (funcs upto 1B)\n");
			exit(2);
        }

		if (fos_info.obufr < 256)
			fprintf(stderr,"!FOSSIL transmit buffer size %u, should be >= 256\n",fos_info.obufr);
		if (fos_info.ibufr < 4096)
			fprintf(stderr,"!FOSSIL receive buffer size %u, should be >= 4096\n",fos_info.ibufr);

		com_flow(flowflags);
        fos_inread = fos_inwrite = 0;
}


void sys_reset(void) /* same as above for de-init */
{
		com_flow(0);
		fossil_int(FOS_DEINIT);
}


int com_outfull (void)       /* Return the amount in fossil output buffer */
{                            /* Ie. remaining no. bytes to be transmitted */
           get_fos_info();
           return (fos_info.obufr - fos_info.ofree);
}

int com_infull (void)        /* Return the amount in fossil input buffer */
{                            /* Ie. remaining no. bytes to be received */
           get_fos_info();
		   return (fos_info.ibufr - fos_info.ifree);
}

void com_flush(void)      /* wait till all characters have been sent */
{
		   while (!(fos_getstatus() & 0x4000));
}


void com_putbyte (byte c)
{
           _AL = c;
           fossil_int(FOS_PUTC);
}


void com_purge (void)
{
           fossil_int(FOS_PURGE);
           fos_inread = fos_inwrite = 0;
}


void com_dump (void)  /* purge outputt buffer */
{
           fossil_int(FOS_DUMP);
}


int com_getbyte(void)
{
		   return (((fos_inread < fos_inwrite) || fos_fillinbuf()) ?
				   fos_inbuf[fos_inread++] : EOF);
}

